@@ -1,5 +1,5 @@
 /* Tarlz - Archiver with multimember lzip compression
-   Copyright (C) 2013-2020 Antonio Diaz Diaz.
+   Copyright (C) 2013-2021 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,26 +20,20 @@
 #include <algorithm>
 #include <cctype>
 #include <cerrno>
-#include <climits>
 #include <cstdio>
 #include <cstdlib>
-#include <cstring>
-#include <string>
-#include <vector>
-#include <pthread.h>	// for tarlz.h
-#include <stdint.h>
+#include <stdint.h>	// for lzlib.h
 #include <unistd.h>
 #include <utime.h>
 #include <sys/stat.h>
-#include <sys/types.h>
 #if !defined __FreeBSD__ && !defined __OpenBSD__ && !defined __NetBSD__ && \
     !defined __DragonFly__ && !defined __APPLE__ && !defined __OS2__
 #include <sys/sysmacros.h>		// for major, minor, makedev
 #endif
 #include <lzlib.h>
 
-#include "arg_parser.h"
 #include "tarlz.h"
+#include "arg_parser.h"
 #include "lzip_index.h"
 #include "archive_reader.h"
 
@@ -455,10 +449,12 @@ int decode( const Cl_options & cl_opts )
 
     extended.fill_from_ustar( header );		// copy metadata from header
 
+    // members without name are skipped except when listing
     if( check_skip_filename( cl_opts, name_pending, extended.path().c_str() ) )
       retval = skip_member( ar, extended );
     else if( cl_opts.program_mode == m_list )
       retval = list_member( ar, extended, header );
+    else if( extended.path().empty() ) retval = skip_member( ar, extended );
     else if( cl_opts.program_mode == m_diff )
       retval = compare_member( cl_opts, ar, extended, header );
     else retval = extract_member( cl_opts, ar, extended, header );
